@@ -153,68 +153,125 @@ It transparently generates nonces and automatically handles key rotation.
 Returns a new secretstream object for the given primitive. The primitive
 argument is required.
 
+=head1 ATTRIBUTES
+
+=head2 primitive
+
+  my $primitive = $sstream->primitive;
+  $sstream->primitive('xchacha20poly1305');
+
+Gets or sets the primitive used for all operations by this object. For this
+module, the returned primitive is always identical to L</PRIMITIVE>.
+
 =head1 METHODS
 
-=head2 PRIMITIVE
-
-  my $default_primitive = $sstream->PRIMITIVE;
-
-=head2 ABYTES
-
-  my $additional_data_length = $sstream->ABYTES;
-
-This is not a restriction on the amount of additional data, it is the size of
-the ciphertext MAC.
-
-=head2 HEADERBYTES
-
-  my $header_length = $sstream->HEADERBYTES;
-
-=head2 KEYBYTES
-
-  my $key_length = $sstream->KEYBYTES;
-
-=head2 MESSAGEBYTES_MAX
-
-  my $message_max_length = $sstream->MESSAGEBYTES_MAX;
-
-=head2 TAG_MESSAGE
-
-  my $message_tag = $sstream->TAG_MESSAGE;
-
-=head2 TAG_PUSH
-
-  my $push_tag = $sstream->TAG_PUSH;
-
-=head2 TAG_REKEY
-
-  my $rekey_tag = $sstream->TAG_REKEY;
-
-=head2 TAG_FINAL
-
-  my $final_tag = $sstream->TAG_FINAL;
-
 =head2 primitives
 
   my @primitives = $sstream->primitives;
 
 Returns a list of all supported primitive names.
 
+Can be called as a class method.
+
+=head2 PRIMITIVE
+
+  my $primitive = $sstream->PRIMITIVE;
+
+Returns the primitive used for all operations by this object. For this module,
+always identical to the L</primitive> attribute.
+
 =head2 init_decrypt
 
-  my $sstream_dec = $sstream->init_decrypt($header, $key);
+  my $sstream_dec = $sstream->init_decrypt($header, $key, $flags);
+
+C<$header> is an opaque header of L</HEADERBYTES> bytes generated by a previous
+call to L</init_encrypt>.
+
+C<$key> is a shared secret key. It must be L</KEYBYTES> bytes. It may be a
+L</Crypt::Sodium::XS::MemVault>.
+
+C<$flags> is optional. It is the flags used for the secretstream decryption
+protected memory object. See L<Crypt::Sodium::XS::ProtMem>.
+
+Returns an opaque protected memory object: a secretstream decryption object.
 
 See L</STREAM INTERFACE>.
 
 =head2 init_encrypt
 
-  my ($header, $sstream_enc) = $sstream->init_encrypt($key);
+  my ($header, $sstream_enc) = $sstream->init_encrypt($key, $flags);
+
+C<$key> is a shared secret key. It must be L</KEYBYTES> bytes. It may be a
+L</Crypt::Sodium::XS::MemVault>.
+
+C<$flags> is optional. It is the flags used for the secretstream encryption
+protected memory object. See L<Crypt::Sodium::XS::ProtMem>.
+
+Returns an opaque header of L</HEADERBYTES> bytes and an opaque protected
+memory object: a secretstream encryption object.
 
 See L</STREAM INTERFACE>.
 
 =head2 keygen
 
-  my $key = $sstream->keygen;
+  my $key = $sstream->keygen($flags);
+
+C<$flags> is optional. It is the flags used for the C<$key>
+L<Crypt::Sodium::XS::MemVault>. See L<Crypt::Sodium::XS::ProtMem>.
+
+Returns a L<Crypt::Sodium::XS::MemVault>: a new secret key of L</KEYBYTES>
+bytes.
+
+=head2 ABYTES
+
+  my $tag_size = $sstream->ABYTES;
+
+Returns the size, in bytes, of an authentication tag. Note that this is not a
+restriction on the amount of additional data (adata).
+
+The size of any ciphertext is message size + L</ABYTES>.
+
+=head2 HEADERBYTES
+
+  my $header_size = $sstream->HEADERBYTES;
+
+Returns the size, in bytes, of a header generated by L</init_encrypt>.
+
+=head2 KEYBYTES
+
+  my $key_size = $sstream->KEYBYTES;
+
+Returns the size, in bytes, of a secret key.
+
+=head2 MESSAGEBYTES_MAX
+
+  my $message_max_size = $sstream->MESSAGEBYTES_MAX;
+
+Returns the size, in bytes, of the maximum size of any message to be encrypted.
+
+=head2 TAG_MESSAGE
+
+  my $message_tag = $sstream->TAG_MESSAGE;
+
+Returns the tag value for a TAG_MESSAGE secretstream tag.
+
+=head2 TAG_PUSH
+
+  my $push_tag = $sstream->TAG_PUSH;
+
+Returns the tag value for a TAG_PUSH secretstream tag.
+
+=head2 TAG_REKEY
+
+  my $rekey_tag = $sstream->TAG_REKEY;
+
+Returns the tag value for a TAG_REKEY secretstream tag.
+
+=head2 TAG_FINAL
+
+  my $final_tag = $sstream->TAG_FINAL;
+
+Returns the tag value for a TAG_FINAL secretstream tag.
 
 =head1 STREAM INTERFACE
 
@@ -250,7 +307,7 @@ about sequence boundaries (PUSH and REKEY tags).
 
 For each message, additional data can be included in the computation of the
 authentication tag. With this API, additional data is rarely required, and most
-applications can just use NULL and a length of 0 instead.
+applications can just omit it.
 
 =head2 ENCRYPTION
 
@@ -264,13 +321,19 @@ methods:
 
   my $ciphertext = $sstream_enc->encrypt($plaintext, $tag, $adata);
 
-C<$tag> is optional, and defaults to L</TAG_MESSAGE>. The most common use is a
-tag of L</TAG_FINAL> to indicate the last message in a stream.
+C<$plaintext> is the plaintext to encrypt. It may be a
+L<Crypt::Sodium::XS::MemVault>.
+
+C<$tag> is optional. It is the tag value attached to the message. If not
+provided, it defaults to L</TAG_MESSAGE>. The most common use is a tag of
+L</TAG_FINAL> to indicate the last message in a stream. See L</OVERVIEW>.
 
 C<$adata> is optional. If provided, it must match the additional data that was
 used when encrypting this message. It is rarely needed with the secretstream
 interface.
 
+Returns the encrypted ciphertext.
+
 =back
 
 =head2 DECRYPTION
@@ -278,10 +341,8 @@ interface.
 The L</init_decrypt> method is the decryption counterpart for the receiving end
 of a stream. It takes a header and a secret key; the key must match the one
 used to create the encryption object, and the header must match the one that
-was returned when it was created.
-
-Returns a secretstream decryption object. This is an opaque object with the
-following methods:
+was returned when it was created. It returns a secretstream decryption object.
+This is an opaque object with the following methods:
 
 =over 4
 
@@ -292,12 +353,13 @@ following methods:
 
 Croaks on decryption failure.
 
-C<$tag> will be one of the tags listed in L</CONSTANTS>; the tag used when
-encrypting this message. The most common use is a tag of
-L</TAG_FINAL> indicating the last message of a stream.
+C<$ciphertext> is the ciphertext to decrypt.
 
 C<$adata> is optional. It is rarely needed with the secretstream interface.
 
+Returns a L<Crypt::Sodium::XS::MemVault>: the decrypted plaintext and the
+message tag. See L</OVERVIEW>.
+
 =back
 
 =head1 SEE ALSO
